/*  AxMl.h  */

#define headerName     "AxMl.h"
#define headerVersion  "1.0.0"
#define headerDate     "February, 2002"


#ifndef dnaml_h

/*  Compile time switches for various updates to program:
 *    0 gives original version
 *    1 gives new version
 */

#define ReturnSmoothedView    1  /* Propagate changes back after smooth */
#define BestInsertAverage     1  /* Build three taxon tree analytically */
#define DeleteCheckpointFile  0  /* Remove checkpoint file when done */

#define EQUALITIES 16


/*  Program constants and parameters  */

#define maxlogf       1024  /* maximum number of user trees */
#define maxcategories   35  /* maximum number of site types */

#define smoothings      32  /* maximum smoothing passes through tree */
#define iterations      10  /* maximum iterations of makenewz per insert */
#define newzpercycle     1  /* iterations of makenewz per tree traversal */
#define nmlngth         100  /* number of characters in species name */
#define deltaz     0.00001  /* test of net branch length change in update */
#define zmin       1.0E-15  /* max branch prop. to -log(zmin) (= 34) */
#define zmax (1.0 - 1.0E-6) /* min branch prop. to 1.0-zmax (= 1.0E-6) */
#define defaultz       0.9  /* value of z assigned as starting point */
#define unlikely  -1.0E300  /* low likelihood for initialization */

/*  These values are used to rescale the lilelihoods at a given site so that
 *  there is no floating point underflow.
 */
#define twotothe256  \
115792089237316195423570985008687907853269984665640564039457584007913129639936.0
                                                     /*  2**256 (exactly)  */
#define minlikelihood     (1.0/twotothe256)          /*  2**(-256)  */
#define log_minlikelihood (-177.445678223345993274)  /* log(1.0/twotothe256) */

/*  The next two values are used for scaling the tree that is sketched in the
 *  output file.
 */
#define down             2
#define over            60

#define checkpointname "checkpoint"

#define badEval        1.0
#define badZ           0.0
#define badRear         -1
#define badSigma      -1.0

#define TRUE             1
#define FALSE            0

#define treeNone         0
#define treeNewick       1
#define treeProlog       2
#define treePHYLIP       3
#define treeMaxType      3
#define treeDefType  treePHYLIP

#define ABS(x)    (((x)<0)   ?  (-(x)) : (x))
#define MIN(x,y)  (((x)<(y)) ?    (x)  : (y))
#define MAX(x,y)  (((x)>(y)) ?    (x)  : (y))
#define LOG(x)    (((x)>0)   ? log(x)  : hang("log domain error"))
#define NINT(x)   ((int) ((x)>0 ? ((x)+0.5) : ((x)-0.5)))


typedef  char  yType;



typedef  int     boolean;
typedef  double  xtype;

typedef  struct  likelihood_vector {
    xtype        a, c, g, t;
    long         exp;
    } likelivector;

typedef struct ratec 
{
  double accumulatedSiteLikelihood;
  double rate;
} rateCategorize;




typedef  struct  xmantyp {
    struct xmantyp  *prev;
    struct xmantyp  *next;
    struct noderec  *owner;
    likelivector    *lv;
    } xarray;






typedef struct fgh{
  double a, c, g, t;
  long exp;
  boolean set;
} homType;




typedef  struct noderec {
  /*Tree divisor*/
  double distance;
  int set;
  /* tree divisor*/
    double           z, z0;
    struct noderec  *next;
    struct noderec  *back;
    int              number;
    xarray          *x;   
    char             name[nmlngth+1]; /*  Space for null termination  */
    yType           *tip;             /*  Pointer to sequence data  */

  /* AxML modification start*/

  char             *equalityVector; /* subtree equality vector for this node*/  
  homType mlv[EQUALITIES];  
  /* AxML modification end*/
  

    } node, *nodeptr;






typedef struct {
  double sumaq; 
  double sumgq; 
  double sumcq; 
  double sumtq;  
  int exp;
} poutsa;

typedef struct {
  double fx1a, fx1g, fx1c, fx1t, sumag, sumct, sumagct;
  int exp;
} memP;

typedef struct {
  double a, b, c, w;
} mnzMem;


typedef struct {
  double a, b, c, w, dlnLidlz, d2lnLdlz2;
} mnzMem2;



typedef struct {
  double sumfx2rfx2y, fx2r, fx2y;
  double a, c, g, t;
  int exp;
} memQ;

typedef  struct {
  int              numsp;       /* number of species (also tr->mxtips) */
  int              sites;       /* number of input sequence positions */
  yType          **y;           /* sequence data array */
  yType            *y0;  
  boolean          freqread;    /* user base frequencies have been read */
  

  
  double freqaREV[maxcategories+1], 
    freqcREV[maxcategories+1], 
    freqgREV[maxcategories+1], 
    freqtREV[maxcategories+1], 
    ttratioREV[maxcategories+1];
  boolean matread;
  double EIGV[maxcategories+1][4][4];
  double invfreq[maxcategories+1][4];
  double EIGN[maxcategories+1][4];
  double fracchangeav;
  double initialRates[5];
  double           freqa, freqc, freqg, freqt, ttratio;
  double          freqr, freqy, invfreqr, invfreqy,
                  freqar, freqcy, freqgr, freqty;
  double           xi, xv, fracchange; /* transition/transversion */
/* End of DNA specific values */
    int             *wgt;         /* weight per sequence pos */
    int             *wgt2;        /* weight per pos (booted) */
    int              categs;      /* number of rate categories */
    } rawdata;

typedef  struct {
  int             *alias;       /* site representing a pattern */
  int             *aliaswgt;    /* weight by pattern */
  int             *rateCategory;
  int              endsite;     /* # of sequence patterns */
  int              endsiteNormal;
  int              wgtsum;      /* sum of weights of positions */
  double          *patrat;      /* rates per pattern */
  double          *patratStored;
  double          *wr;          /* weighted rate per pattern */
  double          *wr2;         /* weight*rate**2 per pattern */
} cruncheddata;

typedef  struct {
    double           likelihood;   
    node           **nodep;
    node            *start;
    node            *outgrnode;
    int              mxtips;
    int              ntips;
    int              nextnode;
    int              opt_level;   
    int              global;      /* branches to cross in full tree */
    int              partswap;    /* branches to cross in partial tree */
    int              outgr;       /* sequence number to use in rooting tree */
  int              NumberOfCategories;
    boolean          prelabeled;  /* the possible tip names are known */
    boolean          smoothed;
    boolean          rooted;
    boolean          userlen;     /* use user-supplied branch lengths */
    rawdata         *rdta;        /* raw data structure */
    cruncheddata    *cdta;        /* crunched data structure */
    double t_value;
    } tree;


#define TREE_INFERENCE 1
#define TREE_EVALUATION 2
#define ALIGNMENT_DIVISION 3
#define SEQUENCE_ADDITION 4
#define FAST_MODE 5

typedef struct conntyp {
    double           z;           /* branch length */
    node            *p, *q;       /* parent and child sectors */
    void            *valptr;      /* pointer to value of subtree */
    int              descend;     /* pointer to first connect of child */
    int              sibling;     /* next connect from same parent */
    } connect, *connptr;

typedef  struct {
    double           likelihood;
   
    connect         *links;       /* pointer to first connect (start) */
    node            *start;
    int              nextlink;    /* index of next available connect */
                                  /* tr->start = tpl->links->p */
    int              ntips;
    int              nextnode;
    int              opt_level;   /* degree of branch swapping explored */
    int              scrNum;      /* position in sorted list of scores */
    int              tplNum;      /* position in sorted list of trees */
    
    boolean          prelabeled;  /* the possible tip names are known */
    boolean          smoothed;    /* branch optimization converged? */
    } topol;

typedef struct {
    double           best;        /* highest score saved */
    double           worst;       /* lowest score saved */
    topol           *start;       /* starting tree for optimization */
    topol          **byScore;
    topol          **byTopol;
    int              nkeep;       /* maximum topologies to save */
    int              nvalid;      /* number of topologies saved */
    int              ninit;       /* number of topologies initialized */
    int              numtrees;    /* number of alternatives tested */
    boolean          improved;
    } bestlist;

typedef  struct {
  double timeLimit;
  boolean          autoparams;
  int categories;
  int              max_rearrange;
  int              stepwidth;
  int              overlap;
  int              initial;
  int              sets;
  int              depth; 
  boolean          optimizeTT;
  int              mode;
  long             boot;        /* bootstrap random number seed */    
  boolean          empf;        /* use empirical base frequencies */
  boolean          interleaved; /* input data are in interleaved format */    
  int              numutrees;   /* number of user trees to read */  
  boolean          restart;     /* resume addition to partial tree */
  boolean          root;        /* use user-supplied outgroup */  
  boolean          usertree;    /* use user-supplied trees */
  boolean          userwgt;     /* use user-supplied position weight mask */
} analdef;

void  exit();



#define dnaml_h
#endif  /* #if undef dnaml_h */
